@@ -1,6 +1,7 @@
 /*
  * Copyright (c) 2004-2009 Voltaire Inc.  All rights reserved.
  * Copyright (c) 2010,2011 Mellanox Technologies LTD.  All rights reserved.
+ * Copyright (c) 2011,2016 Oracle and/or its affiliates. All rights reserved.
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -384,6 +385,7 @@ int main(int argc, char **argv)
 	int selfport = 0;
 	int changed = 0;
 	int i;
+	uint32_t vendorid, rem_vendorid;
 	uint16_t devid, rem_devid;
 	uint64_t val;
 	char *endp;
@@ -515,6 +517,7 @@ int main(int argc, char **argv)
 		port_op = QUERY;
 
 	is_switch = get_node_info(&portid, data);
+	vendorid = (uint32_t) mad_get_field(data, 0, IB_NODE_VENDORID_F);
 	devid = (uint16_t) mad_get_field(data, 0, IB_NODE_DEVID_F);
 
 	if ((port_args[MKEY].set || port_args[MKEYLEASE].set ||
@@ -522,12 +525,12 @@ int main(int argc, char **argv)
 		IBEXIT("Can't set M_Key fields on switch port != 0");
 
 	if (port_op != QUERY || changed)
-		printf("Initial %s PortInfo:\n", is_switch ? "Switch" : "CA");
+		printf("Initial %s PortInfo:\n", is_switch ? "Switch" : "CA/RT");
 	else
-		printf("%s PortInfo:\n", is_switch ? "Switch" : "CA");
+		printf("%s PortInfo:\n", is_switch ? "Switch" : "CA/RT");
 	espeed_cap = get_port_info(&portid, data, portnum, is_switch);
 	show_port_info(&portid, data, portnum, espeed_cap, is_switch);
-	if (is_mlnx_ext_port_info_supported(devid)) {
+	if (is_mlnx_ext_port_info_supported(vendorid, devid)) {
 		get_mlnx_ext_port_info(&portid, data2, portnum);
 		show_mlnx_ext_port_info(&portid, data2, portnum);
 	}
@@ -655,18 +658,26 @@ int main(int argc, char **argv)
 
 			/* Setup portid for peer port */
 			memcpy(&peerportid, &portid, sizeof(peerportid));
-			peerportid.drpath.cnt = 1;
-			peerportid.drpath.p[1] = (uint8_t) portnum;
+			if (portid.lid == 0) {
+				peerportid.drpath.cnt++;
+				if (peerportid.drpath.cnt == IB_SUBNET_PATH_HOPS_MAX) {
+					IBEXIT("Too many hops");
+				}
+			} else {
+				peerportid.drpath.cnt = 1;
 
-			/* Set DrSLID to local lid */
-			if (resolve_self(ibd_ca, ibd_ca_port, &selfportid,
-						&selfport, 0) < 0)
-				IBEXIT("could not resolve self");
-			peerportid.drpath.drslid = (uint16_t) selfportid.lid;
-			peerportid.drpath.drdlid = 0xffff;
+				/* Set DrSLID to local lid */
+				if (resolve_self(ibd_ca, ibd_ca_port, &selfportid,
+						         &selfport, 0) < 0)
+					IBEXIT("could not resolve self");
+				peerportid.drpath.drslid = (uint16_t) selfportid.lid;
+				peerportid.drpath.drdlid = 0xffff;
+			}
+			peerportid.drpath.p[peerportid.drpath.cnt] = (uint8_t) portnum;
 
 			/* Get peer port NodeInfo to obtain peer port number */
 			is_peer_switch = get_node_info(&peerportid, data);
+			rem_vendorid = (uint32_t) mad_get_field(data, 0, IB_NODE_VENDORID_F);
 			rem_devid = (uint16_t) mad_get_field(data, 0, IB_NODE_DEVID_F);
 
 			mad_decode_field(data, IB_NODE_LOCAL_PORT_F,
@@ -677,12 +688,12 @@ int main(int argc, char **argv)
 			peer_espeed_cap = get_port_info(&peerportid, data,
 							peerlocalportnum,
 							is_peer_switch);
-			if (is_mlnx_ext_port_info_supported(rem_devid))
+			if (is_mlnx_ext_port_info_supported(rem_vendorid, rem_devid))
 				get_mlnx_ext_port_info(&peerportid, data2,
 						       peerlocalportnum);
 			show_port_info(&peerportid, data, peerlocalportnum,
 				       peer_espeed_cap, is_peer_switch);
-			if (is_mlnx_ext_port_info_supported(rem_devid))
+			if (is_mlnx_ext_port_info_supported(rem_vendorid, rem_devid))
 				show_mlnx_ext_port_info(&peerportid, data2,
 							peerlocalportnum);
 
